//////////////////////////////////////////////////////////////////////
//
// IBM Confidential
//
// OCO Source Materials
//
// The source code for this program is not published or otherwise
// divested of its trade secrets, irrespective of what has been
// deposited with the U.S. Copyright Office.
//
// 5769-SS1
// (C) Copyright IBM Corp. 1999, 2000
//
//////////////////////////////////////////////////////////////////////
//
// File Name: ClParmListPanel.java
//
// Description: Handles lists in PARM objects.
//
// Class Name:  ClParmListPanel
//
// Purpose:                                                         
// 
//
/////////////////////////////////////////////////////////////////////
//
// CHANGE ACTIVITY:
//
//   $A0=PTR/DCR    Release    Date       Userid    Comments
//       D97942     v5r1m0    07/26/99    BWOOD     New part
//   $A1=P9911169   v5r1m0    12/06/99    BWOOD     Disable add button when field is blank
//   $A2=P9916938   v5r1m0    02/18/00    BWOOD     Fix panel shifting when button pressed
//   $A3=P9928800   v5r1m0    07/19/00    BWOOD     Handle spcvalues properly when restricted
//   $A4=P9928800   v5r1m0    07/21/00    BWOOD     Do not enable controls if entire panel is disabled
//   $A5=P9935722   v5r1m0    10/19/00    BWOOD     Workaround jdk1.3 issue where add button not clickable
//                                                  after being enabled from a JCombobox.
//   $A6=P9945846   v5r1m0f   03/24/01    BWOOD     Fix Turkish i conversion problem
//   $A7=           v5r1m0f   07/12/01    BWOOD     Stop disallowing duplicate entries in list
//
// END CHANGE ACTIVITY
//
////////////////////////////////////////////////////////////////////

package com.ibm.as400.ui.util;

import com.ibm.as400.ui.framework.java.*;

import java.awt.event.*;
import java.awt.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.event.*;


class ClParmListPanel extends ClPanel 
                           implements ActionListener, ItemListener,
                                      ListSelectionListener, DocumentListener
{
   private JButton m_add;
   private JButton m_remove;
   private JButton m_moveup;
   private JButton m_movedown;
   private ClList  m_list;
     
   // values put into the list
   private Vector  m_listData;
   
   // values which are single values
   private Vector  m_sngValues;
   private ClValues m_choicePgmValues;
   
   private JComponent m_entryField;
   private JScrollPane m_scrollList;
   private int m_max;
   private boolean m_bVisible;
   private ClCommonLayout m_parent;
   private boolean m_bRestricted;
   
   ClParmListPanel(JComponent jt, int max, ClCommonLayout cl) throws ClParseException
   {
    this(jt, max, null, null, cl);
   }
    
   // Constructor 
   ClParmListPanel(JComponent jt, int max, ClValues sngValues, ClValues choicePgmValues, ClCommonLayout cl) throws ClParseException
   {     
        setLayout(new GridBagLayout()); 
        
        m_max = max;
        m_entryField = jt;
        m_bVisible = true;
        m_listData = new Vector();
        m_parent = cl;
        m_bRestricted = cl.getRstd();
        
        
        // Set the preferred size so most fields line up on right edge
        ((JTextField)ClCommonLayout.m_tempBox.getEditor().getEditorComponent()).setColumns(ClPanel.getColumns(cl.getLen1()));                
        Dimension d = ClCommonLayout.m_tempBox.getPreferredSize();
        
        if(m_entryField instanceof JComboBox || m_entryField instanceof JTextComponent)
        {
            m_entryField.setPreferredSize(d);                        
        }
        else
        {
            Dimension ed = m_entryField.getPreferredSize();
            ed.width = d.width;
            m_entryField.setPreferredSize(ed);            
        }

        // If there are single values possible for this item,
        // hook up state change listener to combo box being used.
        // Trap the selection of single values.
        if(sngValues != null || m_bRestricted)          //@A3C
        {
            if(sngValues != null)                       //@A3A
                m_sngValues = sngValues.getStrings();   
            else                                        //@A3A
                m_sngValues = null;                     //@A3A
                
            ((JComboBox)m_entryField).addItemListener(this);
            ((JTextComponent)((JComboBox)m_entryField).getEditor().getEditorComponent()).getDocument().addDocumentListener(this);
        }
        else 
        {
            m_sngValues = null;
            
            if(m_entryField instanceof JTextComponent)
                ((JTextField)m_entryField).getDocument().addDocumentListener(this);
            else if(m_entryField instanceof ClTextArea)
                ((ClTextArea)m_entryField).getTextComponent().getDocument().addDocumentListener(this);            
            else if(m_entryField instanceof JComboBox)
                ((JTextComponent)((JComboBox)m_entryField).getEditor().getEditorComponent()).getDocument().addDocumentListener(this);            
        }
        
        m_list = new ClList(m_listData);
        m_list.setVisibleRowCount(4);
        m_list.setFixedCellWidth(m_entryField.getPreferredSize().width);
          
        // Create a focus listner and add it to each component,
        // so they auto-scroll into view when they get focus.
        ClPanelFocusListener focus = new ClPanelFocusListener();
        
        m_scrollList = new JScrollPane();
        m_scrollList.getViewport().add(m_list);
        m_list.addFocusListener(focus);
        
        // Load the Add and Remove buttons from the MRI
        if(ClPanel.m_addString == null)
            ClPanel.m_addString = ClPanel.loadString("IDS_LIST_ADD");            
        if(ClPanel.m_removeString == null)
            ClPanel.m_removeString = ClPanel.loadString("IDS_LIST_REMOVE");
            
        if(ClPanel.m_moveupString == null)
            ClPanel.m_moveupString = ClPanel.loadString("IDS_LIST_MOVEUP");
        if(ClPanel.m_movedownString == null)
            ClPanel.m_movedownString = ClPanel.loadString("IDS_LIST_MOVEDOWN");
        
        // Create the add and remove buttons using info from MRI
        m_add = new JButton(m_addString);
        m_remove = new JButton(m_removeString);
        m_moveup = new JButton(m_moveupString);
        m_movedown = new JButton(m_movedownString);
        
        // Set fonts on buttons
        m_add.setFont(ClPanel.m_scaledFont);
        m_remove.setFont(ClPanel.m_scaledFont);
        m_moveup.setFont(ClPanel.m_scaledFont);
        m_movedown.setFont(ClPanel.m_scaledFont);        
                
        ClListKeyListener enterKeyListener = new ClListKeyListener(this);
        
        // Add key listener to pick up delete key press on list
        m_list.addKeyListener(enterKeyListener);
        m_list.addListSelectionListener(this);
        
        if(m_entryField instanceof JTextComponent)
        {
            ((JTextComponent)m_entryField).addKeyListener(enterKeyListener);
        }
        else if(m_entryField instanceof ClTextArea)
        {
            ((ClTextArea)m_entryField).getTextComponent().addKeyListener(enterKeyListener);
        }
        else if(m_entryField instanceof JComboBox)
        {
            ((JTextComponent)((JComboBox)m_entryField).getEditor().getEditorComponent()).addKeyListener(enterKeyListener);
        }
        
        // Set the action commands
        m_add.setActionCommand("Add");
        m_remove.setActionCommand("Remove");
        m_moveup.setActionCommand("MoveUp");
        m_movedown.setActionCommand("MoveDown");
        
        m_add.addFocusListener(focus);
        m_remove.addFocusListener(focus);
        m_moveup.addFocusListener(focus);
        m_movedown.addFocusListener(focus);
        
        Dimension addDim = m_add.getPreferredSize();
        Dimension remDim = m_remove.getPreferredSize();
        Dimension muDim = m_moveup.getPreferredSize();
        Dimension mdDim = m_movedown.getPreferredSize();
        
        if(remDim.width > addDim.width)
            addDim.width = remDim.width;
        
        if(mdDim.width > muDim.width)
            muDim.width = mdDim.width;
            
        if(muDim.width > addDim.width)
            addDim.width = muDim.width;
            
        if(remDim.height > addDim.height)
            addDim.height = remDim.height;
        
        if(mdDim.height > muDim.height)
            muDim.height = mdDim.height;
            
        if(muDim.height > addDim.height)
            addDim.height = muDim.height;            
        
        m_add.setPreferredSize(addDim);
        m_remove.setPreferredSize(addDim);
        m_moveup.setPreferredSize(addDim);
        m_movedown.setPreferredSize(addDim);
        
        m_add.addActionListener(this);
        m_remove.addActionListener(this);
        m_moveup.addActionListener(this);
        m_movedown.addActionListener(this);
        
        GridBagConstraints gbc = ClPanel.createGridBagConstraints();       
        gbc.insets = new Insets(0, 0, 0, 0);
        gbc.gridwidth = ClPanel.getGridWidth(ClPanel.getColumns(m_parent.getLen1()));
                       
        add(m_entryField, gbc);     
        
        gbc = ClPanel.createGridBagConstraints();       
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.anchor = GridBagConstraints.SOUTHWEST;
        gbc.insets = new Insets(0, INDENT_LEFT, 0, INDENT_RIGHT);
        
        add(m_add, gbc);              
        
        gbc = ClPanel.createGridBagConstraints();       
        gbc.gridwidth = 1; 
        gbc.gridheight = 3;
        gbc.fill = GridBagConstraints.VERTICAL;
        gbc.insets = new Insets(INDENT_LISTTOP, 0, 0, 0);
        
        add(m_scrollList, gbc);
                
        gbc = ClPanel.createGridBagConstraints();       
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.insets = new Insets(INDENT_LISTTOP, INDENT_LEFT, 0, INDENT_RIGHT);

        add(m_remove, gbc);
        
        gbc = ClPanel.createGridBagConstraints();       
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.insets = new Insets(INDENT_LISTTOP, INDENT_LEFT, 0, INDENT_RIGHT);

        add(m_moveup, gbc);

        gbc = ClPanel.createGridBagConstraints();       
        gbc.gridwidth = GridBagConstraints.REMAINDER;
        gbc.insets = new Insets(INDENT_LISTTOP, INDENT_LEFT, 0, INDENT_RIGHT);

        add(m_movedown, gbc);
        
        
        // remove button starts of disabled (unless items already in list)
        m_remove.setEnabled(false);        
        m_moveup.setEnabled(false);
        m_movedown.setEnabled(false);
        m_list.setEnabled(false);
        
        // Now add the starting values to the list control
        setStartingValue(m_parent.getStartingValue());

        validateControls();
        
        // fake a selection event so controls display
        // correctly before the user takes action.
        if(m_sngValues != null || m_bRestricted)                        //@A3C
            itemStateChanged(new ItemEvent(((JComboBox)m_entryField), 
                        0, m_entryField, ItemEvent.ITEM_STATE_CHANGED));
   }
   
   void setStartingValue(String startingValue) throws ClParseException
   {
        if(startingValue == null)
            return;
            
        // Clear the entry field since default isn't needed
        blankControls();
    
        // parse the starting value for this list.  The list should be 
        // a space delemited listing of items which should be placed into
        // the list control.
        String item = null;                        
        boolean lastItem = false;
        boolean firstItem = true;
        while(startingValue != null && !startingValue.equals(""))
        {
            startingValue = startingValue.trim();
            
// @BW COMMENTOUT           int index = startingValue.indexOf(" ");

//@BW START
                    // Make sure imbedded space is not within quotes
                    int length = startingValue.length();
                    int i = 0;                                        
                    int quote = 0;
                    int index = -1;
                    while(i < length)
                    {
                        char character = startingValue.charAt(i);
                            
                        if(character == '\'')
                        {
                            quote++;
                        }
                        else if(character == ' ')
                        {
                            if(quote % 2 == 0)
                            {
                                index = i;
                                break;
                            }                            
                        }
                        
                        i++;
                    }
//@BW END                    
            
            if(index == -1)
            {
                item = startingValue;
                lastItem = true;
            }
            else
            {                                                               
                item = startingValue.substring(0, index);            
            }
                
            if(item == null)
            {        
                break;
            }
            
            item = ClSyntax.quote(m_parent.getType(), item);
            item = ClSyntax.upperCase(m_parent.getType(), item);
                                                    
            if(item.length() > 0 &&                    
                item.charAt(0) != '*' && 
                ClSyntax.stripQuotes(item, false).length() > m_parent.getLen1() &&
                m_parent.getType() != ClTypes.DEC &&
                m_parent.getType() != ClTypes.HEX &&
                m_parent.getType() != ClTypes.LGL)                  //@A7A
            {
                if(item.length() > 1 &&
                (item.charAt(0) == 'x' || item.charAt(0) == 'X') &&
                item.charAt(1) == '\'')
                {
                    // hex value, pass it through   
                }
                else
                {
                    ClNode node = m_parent;
                    while(!(node instanceof ClParm))
                        node = node.getParent();
                            
                    m_parent.getPanel().displayAS400Message(ClTypes.MESSAGE_VALUE_TOO_LONG,
                                            ((ClParm)node).getKwd(), "\'" + item + "\'", new Integer(m_parent.getLen1()));   
                                                        
                    throw new ClParseException();
                }
            }
            
            // Check if item is a single value if more than one item
            if(isSingleValue(item))
            {    
                // Give error if single value an other values are specified
                if(!firstItem || m_listData.size() > 0 || index != -1)
                {
                    ClNode node = m_parent;
                    while(!(node instanceof ClParm))
                        node = node.getParent();
                                
                    m_parent.getPanel().displayAS400Message(ClTypes.MESSAGE_SPC_VALUE_IN_LIST,
                                            ((ClParm)node).getKwd(), item);   
                                
                    throw new ClParseException();
                }
                
                // only value and single...  Put into entry field.
                if(m_entryField instanceof JTextComponent)
                {
                    ((JTextComponent)m_entryField).setText(item);
                }
                else if(m_entryField instanceof ClTextArea)
                {
                    ((ClTextArea)m_entryField).getTextComponent().setText(item);   
                }
                else if(m_entryField instanceof JComboBox)
                {
                    ClPanel.setComboText((JComboBox)m_entryField, item);
                }                 
                
                validateControls();
                
                // Reset the starting value by reading the values out.  This
                // is used to smooth over differences in how the pop returns
                // information and we present it.  By reseting the starting value
                // to what we have internally, we don't resend the parms to the command
                // line when they haven't actually changed.
                m_parent.setStartingValue(getCLString());
                
                return;
            }                            
            
            // Verify item's correctness
            if(m_entryField instanceof JTextComponent)
            {
                JTextComponent tc = (JTextComponent)m_entryField;
                String text = tc.getText();
                tc.setText(item);
                if(!ClSyntax.verifyTextField(this, tc, m_parent, m_parent.getMin(), true))
                    throw new ClParseException();
                tc.setText(text);
            }
            else if(m_entryField instanceof ClTextArea)
            {
                JTextComponent tc = ((ClTextArea)m_entryField).getTextComponent();
                String text = tc.getText();
                tc.setText(item);                
                if(!ClSyntax.verifyTextField(this, tc, m_parent, m_parent.getMin(), true))
                    throw new ClParseException();
                tc.setText(text);
            }
            else if(m_entryField instanceof JComboBox)
            {
                String text = ClPanel.getComboText((JComboBox)m_entryField);
                ClPanel.setComboText((JComboBox)m_entryField, item);
                if(!ClSyntax.verifyComboBox(this, (JComboBox)m_entryField, m_parent, m_parent.getMin(), true))
                    throw new ClParseException();
                ClPanel.setComboText((JComboBox)m_entryField, text);
            }                    
            
            firstItem = false;
                
            // Check if item is already in the list
//            if(isAlreadyInList(m_listData, item, m_parent))       //@A7C
//            {                                                     //@A7C
//                throw new ClParseException();                     //@A7C
//            }                                                     //@A7C
                
            m_listData.addElement(item);
                
            // Check if list items exceed number allowed
            if(m_listData.size() > m_parent.getMax())                   //@BW
            {
                ClNode node = m_parent;
                while(!(node instanceof ClParm))
                    node = node.getParent();
                    
                m_parent.getPanel().displayAS400Message(ClTypes.MESSAGE_TOO_MANY_VALUES,
                                        ((ClParm)node).getKwd(), "");   
                   
                throw new ClParseException(); 
            }
            
            if(index == -1)
                break;
                
            startingValue = startingValue.substring(index+1, startingValue.length());            
        }
        
        // Add elements to the list
        m_list.setListData(m_listData);
        m_scrollList.revalidate();
        m_scrollList.repaint();        
        
        validateControls();
        
        // Reset the starting value by reading the values out.  This
        // is used to smooth over differences in how the pop returns
        // information and we present it.  By reseting the starting value
        // to what we have internally, we don't resend the parms to the command
        // line when they haven't actually changed.
        m_parent.setStartingValue(getCLString());
   }
   
   public void setVisible(boolean visible)
   {
        m_bVisible = visible;    

        m_entryField.setVisible(visible);
        
        super.setVisible(visible);
   }
   
   boolean verify()
   {
    return verify(true);
   }
   
   boolean verify(boolean checkMin)
   {
        // if this item isn't viewable, don't check it
        if(!m_bVisible)
            return true;
            
        // only verifies entry fields...  Will not verify lists.  The
        // entries should have been verified before being added to the list.
        int size = m_listData.size();
        
        // if we don't wish to check the min (if we are just verifying the values
        // so we can place them in the list), set the min to the min...
        if(checkMin == false)
        {
            size = m_parent.getMin();
        }
        else
        {
            String text = null;
            if(m_entryField instanceof JTextComponent)
            {
                text = ((JTextComponent)m_entryField).getText();
            }
            else if(m_entryField instanceof ClTextArea)
            {
                text = ((ClTextArea)m_entryField).getTextComponent().getText();
            }
            else if(m_entryField instanceof JComboBox)
            {
                text = ClPanel.getComboText((JComboBox)m_entryField);  
                
                // Check for choice pgm and map value if needed
                text = ClSyntax.getChoicePgmValue(text, m_parent.getChoicePgmValues());                
            }
            
            if(text != null && !text.equals(""))
            {
                text = ClSyntax.quote(m_parent.getType(), text);
                text = ClSyntax.upperCase(m_parent.getType(), text);
                                        
/*@A7C          if(isAlreadyInList(m_listData, text, m_parent))
                {  
                    if(m_entryField instanceof JComboBox)
                        comboRequestFocus((JComboBox)m_entryField);
                    else
                        m_entryField.requestFocus();
                        
                    return false;
                }
*/               
            }
        }
        
        if(m_entryField instanceof JTextComponent)
        {   
            return ClSyntax.verifyTextField((ClPanel)getParent(), (JTextComponent)m_entryField, m_parent, size, checkMin);
        }
        else if(m_entryField instanceof ClTextArea)
        {
            return ClSyntax.verifyTextField((ClPanel)getParent(), ((ClTextArea)m_entryField).getTextComponent(), m_parent, size, checkMin);
        }
        else if(m_entryField instanceof JComboBox)
        {
            return ClSyntax.verifyComboBox((ClPanel)getParent(), (JComboBox)m_entryField, m_parent, size, checkMin);                                    
        }        
        
        return true;
   }
     
   String getCLString()
   {
        String text = "";
        
        if(m_entryField instanceof JTextComponent)
        {
            text = ((JTextComponent)m_entryField).getText();
        }
        else if(m_entryField instanceof ClTextArea)
        {
            text = ((ClTextArea)m_entryField).getTextComponent().getText();   
        }
        else if(m_entryField instanceof JComboBox)
        {
            text = ClPanel.getComboText((JComboBox)m_entryField);
            
            // If choiceprogramvalues available, check to see if
            // this item is one and map to the shortname for
            // the command line.
            text = ClSyntax.getChoicePgmValue(text, m_choicePgmValues);
        }               
        
        text = ClSyntax.quote(m_parent.getType(), text);
        text = ClSyntax.upperCase(m_parent.getType(), text);
        
        if(m_list.m_enabled)
        {
            int size = m_listData.size();
            
            if(size > 0)
            {
                String listItem = "";
                for(int i = 0; i < size; i++)
                {
                    listItem += m_listData.elementAt(i) + " ";
                }
                text = listItem + text;
            }
        }
        
        return text.trim();
   }   
   
   // compare values are always the first value in the list
   String getCompareValue()
   {
        int size = m_listData.size();
        
        if(size == 0)
            return getCLString();
        else
        {
            // Check if field is filled.  If it is, check to see if single value.
            // If it is, return it.  Otherwise, return the first item in the list.
            String entryText = "";            
            if(m_entryField instanceof JTextComponent)
            {
                entryText = ((JTextComponent)m_entryField).getText();
            }
            else if(m_entryField instanceof ClTextArea)
            {
                entryText = ((ClTextArea)m_entryField).getTextComponent().getText();   
            }
            else if(m_entryField instanceof JComboBox)
            {
                entryText = ClPanel.getComboText((JComboBox)m_entryField);
                
                // If choiceprogramvalues available, check to see if
                // this item is one and map to the shortname for
                // the command line.
                entryText = ClSyntax.getChoicePgmValue(entryText, m_choicePgmValues);
            }               
            
            entryText = entryText.trim();
            if(!entryText.equals(""))
            {
                if(isSingleValue(entryText))   
                    return entryText;
            }
            
            return (String)m_listData.firstElement();
        }
   }
   
   // Assign focus to the first field on this panel.  This will always
   // be the entry field.
   void assignFocus()
   {
        if(m_entryField instanceof JComboBox)
        {
            ClPanel.comboRequestFocus((JComboBox)m_entryField);
        }
        else if(m_entryField instanceof ClTextArea)
        {
            ((ClTextArea)m_entryField).getTextComponent().requestFocus();
        }
        else
            m_entryField.requestFocus();
   }
   
   // Blank all controls on this panel.  This will only include the
   // entry field and the list.
   void blankControls()
   {
        if(m_entryField instanceof JTextComponent)
        {
            ((JTextComponent)m_entryField).setText("");            
        }
        else if(m_entryField instanceof ClTextArea)
        {
            ((ClTextArea)m_entryField).getTextComponent().setText("");
        }
        else if(m_entryField instanceof JComboBox)
        {
            ClPanel.setComboText((JComboBox)m_entryField, ""); 
        }
        
        // clear list
        m_listData.removeAllElements();
        m_list.setListData(m_listData);
        m_scrollList.revalidate();
        m_scrollList.repaint();

        validateControls();
   }
   
   // Refresh this list panel.  This requires us to set the entry
   // field back to it's default value and to empty the list.
   void refresh()
   {
        // empty the list and the entry field
        blankControls();
        
        // Set the entry field back to the default value
        String dft = m_parent.getDft();

        if(m_entryField instanceof JTextComponent)
        {
            ((JTextComponent)m_entryField).setText(dft);            
        }
        else if(m_entryField instanceof ClTextArea)
        {
            ((ClTextArea)m_entryField).getTextComponent().setText(dft);
        }
        else if(m_entryField instanceof JComboBox)
        {
            ClPanel.setComboText((JComboBox)m_entryField, dft); 
        }        
   }
    
   public void actionPerformed(ActionEvent evt)
   {
    String arg = evt.getActionCommand();
    
    if(arg.equals("Add"))
    {
        String text = "";
        
        // This action may come from the user pressing Enter in the
        // entry field.  If so, the command may not be appropriate
        // because there is no list to add the entries.  We return
        // if add is not supported.                
        if(m_add == null || !m_add.isEnabled())
            return;
        
        // verify entry before adding to list
        if(!verify(false))
            return;
        
        if(m_entryField instanceof JTextComponent)
        {
            text = ((JTextComponent)m_entryField).getText();
            ((JTextComponent)m_entryField).setText("");            
        }
        else if(m_entryField instanceof ClTextArea)
        {
            text = ((ClTextArea)m_entryField).getTextComponent().getText();
            ((JTextArea)((ClTextArea)m_entryField).getTextComponent()).setText("");
        }
        else if(m_entryField instanceof JComboBox)
        {
            text = ClPanel.getComboText((JComboBox)m_entryField);
            
            // Check for choice pgm and map value if needed
            text = ClSyntax.getChoicePgmValue(text, m_parent.getChoicePgmValues());            
            
            if(((JComboBox)m_entryField).isEditable())
            {
                ClPanel.setComboText((JComboBox)m_entryField, ""); 
            }
            else
            {
                // select next entry in combo box   
                int index = ((JComboBox)m_entryField).getSelectedIndex();
                index++;
                
                if(((JComboBox)m_entryField).getItemCount() > index)
                    ((JComboBox)m_entryField).setSelectedIndex(index);
            }
        }
        
        if(text != null && !text.equals(""))
        {
            text = ClSyntax.quote(m_parent.getType(), text);
            text = ClSyntax.upperCase(m_parent.getType(), text);
                
                
//            if(!isAlreadyInList(m_listData, text, m_parent))  //@A7C 
            {            
                // Insert item after first selected item if there is one
                int selected = m_list.getSelectedIndex();
                if(m_listData.size() == 0 || selected == -1)
                {
                    m_listData.addElement(text);
                }
                else
                    m_listData.insertElementAt(text, selected+1);
                    
                m_list.setListData(m_listData);
                
                if(m_listData.size() == 1 || selected == -1)
                    m_list.setSelectedIndex(m_listData.size() - 1); // select item just added                
                else
                    m_list.setSelectedIndex(selected+1);
                    
                m_list.ensureIndexIsVisible(m_list.getSelectedIndex());
                
                m_scrollList.repaint();
                m_scrollList.revalidate();

                validateControls();
            }
        }
        
        // set the focus back to the entry field after
        // the user presses add.
        assignFocus();
    }
    else if(arg.equals("Remove"))
    {
        int size = m_listData.size();
        ListSelectionModel selectionModel = m_list.getSelectionModel();
        
        // get last selected index in the list
        int lastIndex = selectionModel.getMaxSelectionIndex();  
        if(lastIndex == -1)
            return;
            
        lastIndex++; // increment so not zero based            
        for(int i = size; i >= 0; i--)
        {
            if(selectionModel.isSelectedIndex(i))
            {
                // remove item and decrement our last selected index
                lastIndex--;
                m_listData.removeElementAt(i);
            }
        }
         
         m_list.setListData(m_listData);
         size = m_listData.size();
         
         if(lastIndex >= size)
            lastIndex = size - 1;
            
         if(lastIndex >= 0)
         {
            m_list.setSelectedIndex(lastIndex);
            m_list.ensureIndexIsVisible(m_list.getSelectedIndex());                                
         }
         
         m_scrollList.revalidate();
         m_scrollList.repaint();
         
         // if added all they can, gray out button  
         validateControls();
         
         m_remove.requestFocus();               //@A2A
    }
    else if(arg.equals("MoveUp"))
    {
        int selected = m_list.getSelectedIndex();
        
        if(selected == -1 || selected == 0)
            return;
            
        Object obj = m_listData.elementAt(selected);
        m_listData.removeElementAt(selected);
        m_listData.insertElementAt(obj, selected - 1);
        m_list.setListData(m_listData);
        m_list.setSelectedIndex(selected - 1);        
        m_list.ensureIndexIsVisible(m_list.getSelectedIndex());                                
        
        m_scrollList.revalidate();
        m_scrollList.repaint();
        
        validateControls();
        
        m_moveup.requestFocus();                //@A2A
    }
    else if(arg.equals("MoveDown"))
    {
        int selected = m_list.getSelectedIndex();
        
        if(selected == -1 || selected == (m_listData.size()-1))
            return;
            
        Object obj = m_listData.elementAt(selected);
        m_listData.removeElementAt(selected);
        m_listData.insertElementAt(obj, selected + 1);
        m_list.setListData(m_listData);
        m_list.setSelectedIndex(selected + 1);        
        m_list.ensureIndexIsVisible(m_list.getSelectedIndex());                            
        
        m_scrollList.revalidate();
        m_scrollList.repaint();
        
        validateControls();
        
        m_movedown.requestFocus();                  //@A2A
    }
   }

   public void changedUpdate(DocumentEvent e)
   {
    
   }
   
   public void insertUpdate(DocumentEvent e)
   {
    documentChanged();
   }

   public void removeUpdate(DocumentEvent e)
   {
    documentChanged();
   }
   
   public void documentChanged()
   {
        // The following runnable was added to fix JDK1.3.  The //@A5A
        // validateControls call was not working correctly when //@A5A
        // it was invoked as a result of document changes in a  //@A5A
        // JCombobox.                                           //@A5A
        class me implements Runnable {                          //@A5A
            public void run() {                                 //@A5A
                validateControls();
            }                                                   //@A5A
        }                                                       //@A5A
                                                                //@A5A
        SwingUtilities.invokeLater(new me());                   //@A5A
   }
   
   public void itemStateChanged(ItemEvent e)
   {
        validateControls();
   }
   
   public void valueChanged(ListSelectionEvent e)
   {
        validateControls();
   }
   
   public void setEnabled(boolean enabled)
   {
        super.setEnabled(enabled);
        
        // Don't blindly enable controls, call the validation routine
        if(enabled)
        {
            validateControls();
        }
        else
        {
            // turn them all off
            m_add.setEnabled(false);
            if(m_entryField instanceof ClTextArea)
            {
                ClPanel.setEnabled(((ClTextArea)m_entryField).getTextComponent(), false);   
            }            
            else
            {
                ClPanel.setEnabled(m_entryField, false);
            }        
            
            m_moveup.setEnabled(false);
            m_movedown.setEnabled(false);
            m_remove.setEnabled(false);
            m_list.setEnabled(false);        
        }
   }
   
   void validateControls()
   {
        // Do not validate if everything is disabled.  Wait to be renabled before taking    //@A4A
        // control of contained controls.  This was causing a problem on the nested         //@A4A
        // elem field of crtlf.                                                             //@A4A
        if(!isEnabled())                                                                    //@A4A
            return;                                                                         //@A4A
            
        if(m_listData.size() >= m_max)
        {
            m_add.setEnabled(false);
            
            if(m_entryField instanceof ClTextArea)
            {
                ClPanel.setEnabled(((ClTextArea)m_entryField).getTextComponent(), false);   
            }            
            else
            {
                ClPanel.setEnabled(m_entryField, false);
            }
        }
        else
        {
            m_add.setEnabled(true);
            if(m_entryField instanceof ClTextArea)
            {
                ClPanel.setEnabled(((ClTextArea)m_entryField).getTextComponent(), true);   
            }            
            else
            {
                ClPanel.setEnabled(m_entryField, true);
            }
        }
         
         if(m_list.getSelectedIndex() != -1)
         {
            if(m_list.getSelectedIndex() != 0)
                m_moveup.setEnabled(m_listData.size() > 1);
            else
                m_moveup.setEnabled(false);
                
            if(m_list.getSelectedIndex() != (m_listData.size() - 1))
                m_movedown.setEnabled(m_listData.size() > 1);                                 
            else
                m_movedown.setEnabled(false);
                
            if(m_listData.size() > 0)    
                m_remove.setEnabled(true);
         }
         else
         {
            m_moveup.setEnabled(false);
            m_movedown.setEnabled(false);
            m_remove.setEnabled(false);
         }
         
         m_list.setEnabled(true);
         
        String text = "";
        
        if(m_entryField instanceof JTextComponent)
        {
            text = ((JTextComponent)m_entryField).getText();
        }
        else if(m_entryField instanceof ClTextArea)
        {
            text = ((ClTextArea)m_entryField).getTextComponent().getText();   
        }
        else if(m_entryField instanceof JComboBox)
        {
            text = ClPanel.getComboText((JComboBox)m_entryField);
        }               
         
        text = text.toUpperCase(Locale.ENGLISH);      //@A3A //@A6C
        text = text.trim();             //@A3A
        
        if(text.equals(""))
            m_add.setEnabled(false);
         
        checkSingleValues(text);
   }    
   
   boolean isSingleValue(String text)
   {
        if(m_sngValues != null)                         //@A3A
        {
            int size = m_sngValues.size();
            
            for(int i = 0; i < size; i++)
            {            
                if(text.equals(m_sngValues.elementAt(i)))       
                {
                    return true;
                }
            }   
        }                                               //@A3A
        
        return false;
   }    
   
   void checkSingleValues(String text)
   {
        if(m_sngValues == null && !m_bRestricted)       //@A3C
            return;
            
        boolean bSingleValueSelected = isSingleValue(text);    
        
        if(bSingleValueSelected)
        {
            m_list.setEnabled(false);
            m_add.setEnabled(false);
            m_remove.setEnabled(false);
            m_moveup.setEnabled(false);
            m_movedown.setEnabled(false);
        }
        else
        {
            if(m_listData.size() > 0)
            {
                m_list.setEnabled(true);
            }            
            
            // If the values are restriced, only enable the add button if a valid
            // value is entered in the combo box.  A valid value is one which is
            // in the drop down.
            if(m_bRestricted)
            {
                // check if text is in the drop down
                int size = ((JComboBox)m_entryField).getItemCount();
                
                for(int i = 0; i < size; i++)
                {
                    if(text.equals(((JComboBox)m_entryField).getItemAt(i)))
                    {
                        m_add.setEnabled(m_listData.size() < m_max);                        
                        return;
                    }
                }
                               
                m_add.setEnabled(false);
            }
            else
            {
               // validateControls();
            }
            
            // Disable add button if entry field contains       //@A1A
            // no value.                                        //@A1A
            if(text.equals("") && m_add != null)                //@A1A
                m_add.setEnabled(false);                        //@A1A
        }
   }    
   
   boolean canUserAdd()
   {
    if(m_add == null)
        return false;
        
    return m_add.isEnabled();
   }
   
    /**
    Returns the copyright.
    **/
    static private String getCopyright()
    {
        return ClCopyright.copyright;
    }      
}

